==> create-fly-1.21.8-6.0.8-2-sources.jar: com/zurrtum/create/client/content/kinetics/chainConveyor/ChainConveyorInteractionHandler.java <==
package com.zurrtum.create.client.content.kinetics.chainConveyor;

import com.google.common.cache.Cache;
import com.zurrtum.create.AllItemTags;
import com.zurrtum.create.AllItems;
import com.zurrtum.create.catnip.data.WorldAttached;
import com.zurrtum.create.catnip.theme.Color;
import com.zurrtum.create.client.catnip.outliner.Outliner;
import com.zurrtum.create.client.content.logistics.packagePort.PackagePortTargetSelectionHandler;
import com.zurrtum.create.client.foundation.utility.RaycastHelper;
import com.zurrtum.create.content.logistics.box.PackageItem;
import com.zurrtum.create.content.logistics.packagePort.PackagePortTarget;
import com.zurrtum.create.foundation.utility.TickBasedCache;
import com.zurrtum.create.infrastructure.packet.c2s.ChainConveyorConnectionPacket;
import com.zurrtum.create.infrastructure.packet.c2s.ChainPackageInteractionPacket;
import java.util.List;
import java.util.Map;
import net.minecraft.class_1799;
import net.minecraft.class_1921;
import net.minecraft.class_2338;
import net.minecraft.class_238;
import net.minecraft.class_239;
import net.minecraft.class_243;
import net.minecraft.class_310;
import net.minecraft.class_4587;
import net.minecraft.class_4588;
import net.minecraft.class_4597;
import net.minecraft.class_5134;
import net.minecraft.class_746;

public class ChainConveyorInteractionHandler {

    public static WorldAttached<Cache<class_2338, List<ChainConveyorShape>>> loadedChains = new WorldAttached<>($ -> new TickBasedCache<>(60, true));

    public static class_2338 selectedLift;
    public static float selectedChainPosition;
    public static class_2338 selectedConnection;
    public static class_243 selectedBakedPosition;
    public static ChainConveyorShape selectedShape;

    public static void clientTick(class_310 mc) {
        if (!isActive(mc)) {
            selectedLift = null;
            return;
        }

        class_746 player = mc.field_1724;
        boolean isWrench = player.method_24520(i -> i.method_31573(AllItemTags.TOOLS_WRENCH));
        boolean dismantling = isWrench && player.method_5715();
        double range = player.method_45325(class_5134.field_47758) + 1;

        class_243 from = player.method_33571();
        class_243 to = RaycastHelper.getTraceTarget(player, range, from);
        class_239 hitResult = mc.field_1765;

        double bestDiff = Float.MAX_VALUE;
        if (hitResult != null)
            bestDiff = hitResult.method_17784().method_1025(from);

        class_2338 bestLift = null;
        ChainConveyorShape bestShape = null;
        selectedConnection = null;

        for (Map.Entry<class_2338, List<ChainConveyorShape>> entry : loadedChains.get(mc.field_1687).asMap().entrySet()) {
            class_2338 liftPos = entry.getKey();
            for (ChainConveyorShape chainConveyorShape : entry.getValue()) {
                if (chainConveyorShape instanceof ChainConveyorShape.ChainConveyorBB && dismantling)
                    continue;
                class_243 liftVec = class_243.method_24954(liftPos);
                class_243 intersect = chainConveyorShape.intersect(from.method_1020(liftVec), to.method_1020(liftVec));
                if (intersect == null)
                    continue;

                double distanceToSqr = intersect.method_1019(liftVec).method_1025(from);
                if (distanceToSqr > bestDiff)
                    continue;
                bestDiff = distanceToSqr;
                bestLift = liftPos;
                bestShape = chainConveyorShape;
                selectedChainPosition = chainConveyorShape.getChainPosition(intersect);
                if (chainConveyorShape instanceof ChainConveyorShape.ChainConveyorOBB obb)
                    selectedConnection = obb.connection;
            }
        }

        selectedLift = bestLift;
        if (bestLift == null)
            return;

        selectedShape = bestShape;
        selectedBakedPosition = bestShape.getVec(bestLift, selectedChainPosition);

        if (!isWrench) {
            Outliner.getInstance().chaseAABB("ChainPointSelection", new class_238(selectedBakedPosition, selectedBakedPosition)).colored(Color.WHITE)
                .lineWidth(1 / 6f).disableLineNormals();
        }
    }

    private static boolean isActive(class_310 mc) {
        class_1799 mainHandItem = mc.field_1724.method_6047();
        return mc.field_1724.method_24520(i -> i.method_31573(AllItemTags.CHAIN_RIDEABLE)) || mainHandItem.method_31574(AllItems.PACKAGE_FROGPORT) || PackageItem.isPackage(
            mainHandItem);
    }

    public static boolean onUse(class_310 mc) {
        if (selectedLift == null)
            return false;

        class_746 player = mc.field_1724;
        class_1799 mainHandItem = player.method_6047();

        if (player.method_24520(i -> i.method_31573(AllItemTags.CHAIN_RIDEABLE))) {
            class_1799 usedItem = mainHandItem.method_31573(AllItemTags.CHAIN_RIDEABLE) ? mainHandItem : player.method_6079();
            if (!player.method_5715()) {
                ChainConveyorRidingHandler.embark(mc, selectedLift, selectedChainPosition, selectedConnection);
                return true;
            }

            player.field_3944.method_52787(new ChainConveyorConnectionPacket(selectedLift, selectedLift.method_10081(selectedConnection), usedItem, false));
            return true;
        }

        if (mainHandItem.method_31574(AllItems.PACKAGE_FROGPORT)) {
            PackagePortTargetSelectionHandler.exactPositionOfTarget = selectedBakedPosition;
            PackagePortTargetSelectionHandler.activePackageTarget = new PackagePortTarget.ChainConveyorFrogportTarget(
                selectedLift,
                selectedChainPosition,
                selectedConnection,
                false
            );
            return true;
        }

        if (PackageItem.isPackage(mainHandItem)) {
            player.field_3944.method_52787(new ChainPackageInteractionPacket(
                selectedLift,
                selectedConnection == null ? class_2338.field_10980 : selectedConnection,
                selectedChainPosition,
                false
            ));
            return true;
        }

        return true;
    }

    public static void drawCustomBlockSelection(class_4587 ms, class_4597 buffer, class_243 camera) {
        if (selectedLift == null || selectedShape == null)
            return;

        class_4588 vb = buffer.getBuffer(class_1921.method_23594());
        ms.method_22903();
        ms.method_22904(selectedLift.method_10263() - camera.field_1352, selectedLift.method_10264() - camera.field_1351, selectedLift.method_10260() - camera.field_1350);
        selectedShape.drawOutline(selectedLift, ms, vb);
        ms.method_22909();
    }

    public static boolean hideVanillaBlockSelection() {
        return selectedLift != null && selectedShape != null;
    }

}

==> create-fly-1.21.10-6.0.8-5-sources.jar: com/zurrtum/create/client/content/kinetics/chainConveyor/ChainConveyorInteractionHandler.java <==
package com.zurrtum.create.client.content.kinetics.chainConveyor;

import com.google.common.cache.Cache;
import com.zurrtum.create.AllItemTags;
import com.zurrtum.create.AllItems;
import com.zurrtum.create.catnip.data.WorldAttached;
import com.zurrtum.create.catnip.theme.Color;
import com.zurrtum.create.client.catnip.outliner.Outliner;
import com.zurrtum.create.client.content.logistics.packagePort.PackagePortTargetSelectionHandler;
import com.zurrtum.create.client.foundation.utility.RaycastHelper;
import com.zurrtum.create.content.logistics.box.PackageItem;
import com.zurrtum.create.content.logistics.packagePort.PackagePortTarget;
import com.zurrtum.create.foundation.utility.TickBasedCache;
import com.zurrtum.create.infrastructure.packet.c2s.ChainConveyorConnectionPacket;
import com.zurrtum.create.infrastructure.packet.c2s.ChainPackageInteractionPacket;
import java.util.List;
import java.util.Map;
import net.minecraft.class_1799;
import net.minecraft.class_1921;
import net.minecraft.class_2338;
import net.minecraft.class_238;
import net.minecraft.class_239;
import net.minecraft.class_243;
import net.minecraft.class_310;
import net.minecraft.class_4587;
import net.minecraft.class_4588;
import net.minecraft.class_4597;
import net.minecraft.class_5134;
import net.minecraft.class_746;

public class ChainConveyorInteractionHandler {

    public static WorldAttached<Cache<class_2338, List<ChainConveyorShape>>> loadedChains = new WorldAttached<>($ -> new TickBasedCache<>(60, true));

    public static class_2338 selectedLift;
    public static float selectedChainPosition;
    public static class_2338 selectedConnection;
    public static class_243 selectedBakedPosition;
    public static ChainConveyorShape selectedShape;

    public static void clientTick(class_310 mc) {
        if (!isActive(mc)) {
            selectedLift = null;
            return;
        }

        class_746 player = mc.field_1724;
        boolean isWrench = player.method_24520(i -> i.method_31573(AllItemTags.TOOLS_WRENCH));
        boolean dismantling = isWrench && player.method_5715();
        double range = player.method_45325(class_5134.field_47758) + 1;

        class_243 from = player.method_33571();
        class_243 to = RaycastHelper.getTraceTarget(player, range, from);
        class_239 hitResult = mc.field_1765;

        double bestDiff = Float.MAX_VALUE;
        if (hitResult != null)
            bestDiff = hitResult.method_17784().method_1025(from);

        class_2338 bestLift = null;
        ChainConveyorShape bestShape = null;
        selectedConnection = null;

        for (Map.Entry<class_2338, List<ChainConveyorShape>> entry : loadedChains.get(mc.field_1687).asMap().entrySet()) {
            class_2338 liftPos = entry.getKey();
            for (ChainConveyorShape chainConveyorShape : entry.getValue()) {
                if (chainConveyorShape instanceof ChainConveyorShape.ChainConveyorBB && dismantling)
                    continue;
                class_243 liftVec = class_243.method_24954(liftPos);
                class_243 intersect = chainConveyorShape.intersect(from.method_1020(liftVec), to.method_1020(liftVec));
                if (intersect == null)
                    continue;

                double distanceToSqr = intersect.method_1019(liftVec).method_1025(from);
                if (distanceToSqr > bestDiff)
                    continue;
                bestDiff = distanceToSqr;
                bestLift = liftPos;
                bestShape = chainConveyorShape;
                selectedChainPosition = chainConveyorShape.getChainPosition(intersect);
                if (chainConveyorShape instanceof ChainConveyorShape.ChainConveyorOBB obb)
                    selectedConnection = obb.connection;
            }
        }

        selectedLift = bestLift;
        if (bestLift == null)
            return;

        selectedShape = bestShape;
        selectedBakedPosition = bestShape.getVec(bestLift, selectedChainPosition);

        if (!isWrench) {
            Outliner.getInstance().chaseAABB("ChainPointSelection", new class_238(selectedBakedPosition, selectedBakedPosition)).colored(Color.WHITE)
                .lineWidth(1 / 6f).disableLineNormals();
        }
    }

    private static boolean isActive(class_310 mc) {
        class_1799 mainHandItem = mc.field_1724.method_6047();
        return mc.field_1724.method_24520(i -> i.method_31573(AllItemTags.CHAIN_RIDEABLE)) || mainHandItem.method_31574(AllItems.PACKAGE_FROGPORT) || PackageItem.isPackage(
            mainHandItem);
    }

    public static boolean onUse(class_310 mc) {
        if (selectedLift == null)
            return false;

        class_746 player = mc.field_1724;
        class_1799 mainHandItem = player.method_6047();

        if (player.method_24520(i -> i.method_31573(AllItemTags.CHAIN_RIDEABLE))) {
            class_1799 usedItem = mainHandItem.method_31573(AllItemTags.CHAIN_RIDEABLE) ? mainHandItem : player.method_6079();
            if (!player.method_5715()) {
                ChainConveyorRidingHandler.embark(mc, selectedLift, selectedChainPosition, selectedConnection);
                return true;
            }

            player.field_3944.method_52787(new ChainConveyorConnectionPacket(selectedLift, selectedLift.method_10081(selectedConnection), usedItem, false));
            return true;
        }

        if (mainHandItem.method_31574(AllItems.PACKAGE_FROGPORT)) {
            PackagePortTargetSelectionHandler.exactPositionOfTarget = selectedBakedPosition;
            PackagePortTargetSelectionHandler.activePackageTarget = new PackagePortTarget.ChainConveyorFrogportTarget(
                selectedLift,
                selectedChainPosition,
                selectedConnection,
                false
            );
            return true;
        }

        if (PackageItem.isPackage(mainHandItem)) {
            player.field_3944.method_52787(new ChainPackageInteractionPacket(
                selectedLift,
                selectedConnection == null ? class_2338.field_10980 : selectedConnection,
                selectedChainPosition,
                false
            ));
            return true;
        }

        return true;
    }

    public static void drawCustomBlockSelection(class_4587 ms, class_4597 buffer, class_243 camera) {
        if (selectedLift == null || selectedShape == null)
            return;

        class_4588 vb = buffer.method_73477(class_1921.method_23594());
        ms.method_22903();
        ms.method_22904(selectedLift.method_10263() - camera.field_1352, selectedLift.method_10264() - camera.field_1351, selectedLift.method_10260() - camera.field_1350);
        selectedShape.drawOutline(selectedLift, ms, vb);
        ms.method_22909();
    }

    public static boolean hideVanillaBlockSelection() {
        return selectedLift != null && selectedShape != null;
    }

}
